/* OdLib_Io - Copyright (c) 2013-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io;

import java.io.IOException;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.MoneyLong;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public final class OdXML {

    private OdXML() {
    }

    public static void openDocumentContent(XMLWriter xmlWriter) throws IOException {
        xmlWriter.startOpenTag("office:document-content");
        xmlWriter.addAttribute("xmlns:office", "urn:oasis:names:tc:opendocument:xmlns:office:1.0");
        xmlWriter.addAttribute("xmlns:style", "urn:oasis:names:tc:opendocument:xmlns:style:1.0");
        xmlWriter.addAttribute("xmlns:text", "urn:oasis:names:tc:opendocument:xmlns:text:1.0");
        xmlWriter.addAttribute("xmlns:table", "urn:oasis:names:tc:opendocument:xmlns:table:1.0");
        xmlWriter.addAttribute("xmlns:draw", "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0");
        xmlWriter.addAttribute("xmlns:fo", "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0");
        xmlWriter.addAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
        xmlWriter.addAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
        xmlWriter.addAttribute("xmlns:meta", "urn:oasis:names:tc:opendocument:xmlns:meta:1.0");
        xmlWriter.addAttribute("xmlns:number", "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0");
        xmlWriter.addAttribute("xmlns:svg", "urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0");
        xmlWriter.addAttribute("xmlns:chart", "urn:oasis:names:tc:opendocument:xmlns:chart:1.0");
        xmlWriter.addAttribute("xmlns:dr3d", "urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0");
        xmlWriter.addAttribute("xmlns:math", "http://www.w3.org/1998/Math/MathML");
        xmlWriter.addAttribute("xmlns:form", "urn:oasis:names:tc:opendocument:xmlns:form:1.0");
        xmlWriter.addAttribute("xmlns:script", "urn:oasis:names:tc:opendocument:xmlns:script:1.0");
        xmlWriter.addAttribute("xmlns:ooo", "http://openoffice.org/2004/office");
        xmlWriter.addAttribute("xmlns:ooow", "http://openoffice.org/2004/writer");
        xmlWriter.addAttribute("xmlns:oooc", "http://openoffice.org/2004/calc");
        xmlWriter.addAttribute("xmlns:dom", "http://www.w3.org/2001/xml-events");
        xmlWriter.addAttribute("xmlns:xforms", "http://www.w3.org/2002/xforms");
        xmlWriter.addAttribute("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");
        xmlWriter.addAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
        xmlWriter.addAttribute("office:version", "1.0");
        xmlWriter.endOpenTag();
    }

    public static void closeDocumentContent(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("office:document-content");
    }

    public static void openAutomaticStyles(XMLWriter xmlWriter) throws IOException {
        xmlWriter.openTag("office:automatic-styles");
    }

    public static void closeAutomaticStyles(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("office:automatic-styles");
    }

    public static void openBody(XMLWriter xmlWriter) throws IOException {
        xmlWriter.openTag("office:body");
    }

    public static void closeBody(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("office:body");
    }

    public static void openSpreadsheet(XMLWriter xmlWriter) throws IOException {
        xmlWriter.openTag("office:spreadsheet");
    }

    public static void closeSpreadsheet(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("office:spreadsheet");
    }

    public static void openStyle(XMLWriter xmlWriter, String styleName, String styleFamily, String parentStyleName) throws IOException {
        startStyleOpenTag(xmlWriter, styleName, styleFamily, parentStyleName);
        xmlWriter.endOpenTag();
    }

    public static void startStyleOpenTag(XMLWriter xmlWriter, String styleName, String styleFamily, String parentStyleName) throws IOException {
        xmlWriter.startOpenTag("style:style");
        xmlWriter.addAttribute("style:name", styleName);
        if (styleName.indexOf("_20_") != -1) {
            String displayName = styleName.replace("_20_", " ");
            xmlWriter.addAttribute("style:display-name", displayName);
        }
        xmlWriter.addAttribute("style:family", styleFamily);
        xmlWriter.addAttribute("style:parent-style-name", parentStyleName);

    }

    public static void closeStyle(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("style:style");
    }

    public static void openTable(XMLWriter xmlWriter, String name) throws IOException {
        xmlWriter.startOpenTag("table:table");
        xmlWriter.addAttribute("table:name", checkTableName(name));
        xmlWriter.endOpenTag();
    }

    public static void closeTable(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("table:table");
    }

    public static void addTableColummn(XMLWriter xmlWriter, String styleName, int columnsRepeated, String defaultCellStyleName) throws IOException {
        xmlWriter.startOpenTag("table:table-column");
        xmlWriter.addAttribute("table:style-name", styleName);
        if (columnsRepeated > 1) {
            xmlWriter.addAttribute("table:number-columns-repeated", columnsRepeated);
        }
        if (defaultCellStyleName == null) {
            defaultCellStyleName = "Default";
        }
        xmlWriter.addAttribute("table:default-cell-style-name", defaultCellStyleName);
        xmlWriter.closeEmptyTag();
    }

    public static void openTableRow(XMLWriter xmlWriter) throws IOException {
        xmlWriter.openTag("table:table-row");
    }

    public static void closeTableRow(XMLWriter xmlWriter) throws IOException {
        xmlWriter.closeTag("table:table-row");
    }

    public static void addEmptyTableCell(XMLWriter xmlWriter) throws IOException {
        xmlWriter.startOpenTag("table:table-cell");
        xmlWriter.closeEmptyTag();
    }

    public static void addNumberTableCell(XMLWriter xmlWriter, int itg) throws IOException {
        addNumberTableCell(xmlWriter, String.valueOf(itg), null, 1);
    }

    public static void addNumberTableCell(XMLWriter xmlWriter, String numberValue) throws IOException {
        addNumberTableCell(xmlWriter, numberValue, null, 1);
    }

    public static void addNumberTableCell(XMLWriter xmlWriter, String numberValue, String cellStyleName, int colSpan) throws IOException {
        xmlWriter.startOpenTag("table:table-cell");
        xmlWriter.addAttribute("office:value-type", "float");
        xmlWriter.addAttribute("office:value", numberValue);
        xmlWriter.addAttribute("table:style-name", cellStyleName);
        if (colSpan > 1) {
            xmlWriter.addAttribute("table:number-columns-spanned", colSpan);
        }
        xmlWriter.closeEmptyTag();
        completeColSpan(xmlWriter, colSpan);
    }

    public static void addDateTableCell(XMLWriter xmlWriter, int dateInteger) throws IOException {
        addDateTableCell(xmlWriter, dateInteger, null, 1);
    }

    public static void addDateTableCell(XMLWriter xmlWriter, int dateInteger, String cellStyleName, int colSpan) throws IOException {
        xmlWriter.startOpenTag("table:table-cell");
        xmlWriter.addAttribute("office:value-type", "date");
        xmlWriter.addAttribute("office:date-value", DateInteger.toISOString(dateInteger));
        xmlWriter.addAttribute("table:style-name", cellStyleName);
        if (colSpan > 1) {
            xmlWriter.addAttribute("table:number-columns-spanned", colSpan);
        }
        xmlWriter.closeEmptyTag();
        completeColSpan(xmlWriter, colSpan);
    }

    public static void addCurrencyTableCell(XMLWriter xmlWriter, long montant, int currencyInteger) throws IOException {
        addCurrencyTableCell(xmlWriter, montant, currencyInteger, null, 1);
    }

    public static void addCurrencyTableCell(XMLWriter xmlWriter, long montant, int currencyInteger, String cellStyleName, int colSpan) throws IOException {
        xmlWriter.startOpenTag("table:table-cell");
        xmlWriter.addAttribute("office:value-type", "currency");
        xmlWriter.addAttribute("office:currency", CurrencyInteger.toString(currencyInteger));
        xmlWriter.addAttribute("office:value", MoneyLong.toDecimal(montant, currencyInteger, false).toString());
        xmlWriter.addAttribute("table:style-name", cellStyleName);
        if (colSpan > 1) {
            xmlWriter.addAttribute("table:number-columns-spanned", colSpan);
        }
        xmlWriter.closeEmptyTag();
        completeColSpan(xmlWriter, colSpan);
    }

    public static void addLinkStringTableCell(XMLWriter xmlWriter, String linkString) throws IOException {
        xmlWriter.startOpenTag("table:table-cell");
        xmlWriter.addAttribute("office:value-type", "string");
        xmlWriter.endOpenTag();
        if ((linkString != null) && (linkString.length() > 0)) {
            xmlWriter.startOpenTag("text:p");
            xmlWriter.endOpenTag();
            addLink(xmlWriter, linkString);
            xmlWriter.closeTag("text:p", false);
        }
        xmlWriter.closeTag("table:table-cell");
    }

    public static void addStringTableCell(XMLWriter xmlWriter, String value) throws IOException {
        addStringTableCell(xmlWriter, value, null, 1);
    }

    public static void addStringTableCell(XMLWriter xmlWriter, String value, String cellStyleName, int colSpan) throws IOException {
        xmlWriter.startOpenTag("table:table-cell");
        xmlWriter.addAttribute("office:value-type", "string");
        xmlWriter.addAttribute("table:style-name", cellStyleName);
        if (colSpan > 1) {
            xmlWriter.addAttribute("table:number-columns-spanned", colSpan);
        }
        xmlWriter.endOpenTag();
        splitText(xmlWriter, value);
        xmlWriter.closeTag("table:table-cell");
        completeColSpan(xmlWriter, colSpan);
    }

    public static void splitText(XMLWriter xmlWriter, String value) throws IOException {
        if ((value != null) && (value.length() > 0)) {
            String[] tokenArray = StringUtils.getTokens(value, '\n', StringUtils.NOTCLEAN);
            int length = tokenArray.length;
            for (int i = 0; i < length; i++) {
                String token = tokenArray[i];
                xmlWriter.startOpenTag("text:p");
                xmlWriter.endOpenTag();
                if (token.length() > 0) {
                    xmlWriter.addText(token);
                }
                xmlWriter.closeTag("text:p", false);
            }
        }
    }

    public static void addLink(XMLWriter xmlWriter, String link) throws IOException {
        xmlWriter.startOpenTag("text:a", false);
        xmlWriter.addAttribute("xlink:href", link);
        xmlWriter.endOpenTag();
        xmlWriter.addText(link);
        xmlWriter.closeTag("text:a", false);
    }

    private static void completeColSpan(XMLWriter xmlWriter, int colSpan) throws IOException {
        if (colSpan > 1) {
            xmlWriter.startOpenTag("table:covered-table-cell");
            xmlWriter.addAttribute("table:number-columns-repeated", (colSpan - 1));
            xmlWriter.closeEmptyTag();
        }
    }

    private static String checkTableName(String name) {
        int length = name.length();
        StringBuilder buf = new StringBuilder(length + 10);
        for (int i = 0; i < length; i++) {
            char carac = name.charAt(i);
            switch (carac) {
                case '[':
                    carac = '(';
                    break;
                case ']':
                    carac = ')';
                    break;
                case '*':
                case ':':
                case '/':
                case '?':
                case '\\':
                    carac = '-';
                    break;
            }
            buf.append(carac);
        }
        return buf.toString();
    }

}
