/* Scrutari - Copyright (c) 2009-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.text.Libelle;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public final class CategoriesPrinter {

    private CategoriesPrinter() {
    }

    public static void printCategories(AdminHtmlPage page) {
        DataAccess dataAccess = page.getSctServer().getScrutariSession().getScrutariDB().openDataAccess();
        GlobalSearchOptions globalSearchOptions = page.getSctServer().getScrutariSession().getGlobalSearchOptions();
        int count = globalSearchOptions.getCategoryCount();
        if (count == 0) {
            page.startP();
            page.startSpan(HA.classes("scrutari-admin-Info"));
            page.localize("_ info.sctconf.none_categories");
            page.endSpan();
            page.endP();
        } else {
            page.startUl();
            for (int i = 0; i < count; i++) {
                Category category = globalSearchOptions.getCategoryByIndex(i);
                printCategoryLi(page, category.getCategoryDef(), dataAccess);
            }
            page.endUl();
        }
        dataAccess.close();
    }

    private static void printCategoryLi(AdminHtmlPage page, CategoryDef categoryDef, DataAccess dataAccess) {
        HTMLAttributes errorAttributes = HA.classes("scrutari-state-Error");
        String name = categoryDef.getName();
        page.startLi();
        page.startP();
        page.startCode();
        page.append("[");
        page.append(String.valueOf(categoryDef.getRank()));
        page.append(" / ");
        page.append(name);
        page.append("]");
        page.endCode();
        page.append(" - ");
        int libelleCount = categoryDef.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            if (i > 0) {
                page.append(" / ");
            }
            Libelle libelle = categoryDef.getLibelleByIndex(i);
            page.escape(libelle.getLibelleString());
        }
        page.endP();
        int uriCount = categoryDef.getCorpusURICount();
        page.startUl();
        if (uriCount == 0) {
            page.startLi();
            page.startP();
            page.startEm();
            if (name.equals("_default")) {
                page.localize("_ info.sctconf.defaultcorpusuri");
            } else {
                page.localize("_ info.sctconf.none_corpusuri");
            }
            page.endEm();
            page.endP();
            page.endLi();
        } else {
            for (int i = 0; i < uriCount; i++) {
                CorpusURI corpusURI = categoryDef.getCorpusURI(i);
                int codecorpus = dataAccess.getCode(corpusURI);
                boolean isHere = (codecorpus > 0);
                page.startLi();
                page.startP();
                if (!isHere) {
                    page.startSpan(errorAttributes);
                }
                page.startCode();
                page.append("[");
                page.append(corpusURI.toString());
                page.append("]");
                page.endCode();
                if (!isHere) {
                    page.endSpan();
                }
                page.endP();
                page.endLi();
            }
        }
        page.endUl();
        page.endLi();
    }

}
