/* Scrutari - Copyright (c) 2009-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.collect.ScrutariSourceList;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import fr.exemole.sctserver.tools.collect.CollectUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public final class SourcesPrinter {

    private SourcesPrinter() {
    }

    public static void printScrutariSourceList(AdminHtmlPage page) {
        ScrutariSourceManager scrutariSourceManager = page.getSctServer().getScrutariInfoManager();
        ScrutariSourceList scrutariSourceList = scrutariSourceManager.getScrutariSourceList();
        int count = scrutariSourceList.getScrutariSourceCount();
        if (count == 0) {
            page.startP();
            page.startSpan(HA.classes("scrutari-admin-Info"));
            page.localize("_ info.sctconf.none_sources");
            page.endSpan();
            page.endP();
            return;
        }
        page.startTable(HA.classes("scrutari-admin-Sources"));
        for (int i = 0; i < count; i++) {
            ScrutariSource scrutariSource = scrutariSourceList.getScrutariSource(i);
            ScrutariSourceDef scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            page.startRow();
            page.startCell();
            page.append(scrutariSourceDef.getName());
            page.endCell();
            short type = scrutariSourceDef.getUrlType();
            page.startCell();
            switch (type) {
                case ScrutariSourceDef.INFO_TYPE:
                    page.append("info");
                    break;
                case ScrutariSourceDef.SCRUTARIDATA_TYPE:
                    page.append("data");
                    break;
            }
            page.endCell();
            page.startCell();
            page.append(scrutariSourceDef.getUrl().toString());
            page.endCell();
            short state = scrutariSource.getState();
            page.startCell();
            page.startSpan(getSpanAttributes(state));
            page.localize(CollectUtils.getStateMessageKey(state));
            page.endSpan();
            page.endCell();
            page.startCell();
            page.append(DateInteger.toISOString(scrutariSource.getLastUpdate()));
            page.endCell();
            page.endRow();
        }
        page.endTable();
    }

    private static HTMLAttributes getSpanAttributes(short state) {
        switch (state) {
            case ScrutariSource.OK_STATE:
                return HA.classes("scrutari-admin-DoneMessage");
            case ScrutariSource.UNKNOWN_STATE:
            case ScrutariSource.INFO_NOCONNECTION_STATE:
            case ScrutariSource.INFO_BROKENURL_STATE:
            case ScrutariSource.INFO_DOWNLOADERROR_STATE:
            case ScrutariSource.INFO_NOTXML_STATE:
            case ScrutariSource.INFO_WRONGXML_STATE:
            case ScrutariSource.DATA_NOCONNECTION_STATE:
            case ScrutariSource.DATA_BROKENURL_STATE:
            case ScrutariSource.DATA_DOWNLOADERROR_STATE:
            case ScrutariSource.DATA_NOTXML_STATE:
                return HA.classes("scrutari-admin-ErrorMessage");
            default:
                throw new SwitchException("state = " + state);
        }
    }

}
