/* Scrutari - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.explore;

import fr.exemole.sctserver.api.SctServer;
import fr.exemole.sctserver.servlets.explore.xml.BaseListXMLPart;
import fr.exemole.sctserver.servlets.explore.xml.LogListXMLPart;
import java.io.IOException;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XmlProducer;
import net.scrutari.db.api.DataAccess;


/**
 *
 * @author Vincent Calame
 */
public class ExploreXmlProducerFactory implements ExploreConstants {

    public static XmlProducer getBaseListXmlProducer(SctServer sctServer) {
        return new BaseListXmlProducer(sctServer);
    }

    public static XmlProducer getLogListXmlProducer(SctServer sctServer) {
        return new LogListXmlProducer(sctServer);
    }


    private static class BaseListXmlProducer implements XmlProducer {

        private SctServer sctServer;

        private BaseListXmlProducer(SctServer sctServer) {
            this.sctServer = sctServer;
        }

        @Override
        public void writeXml(Appendable appendable) throws IOException {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(appendable);
            xmlWriter.appendXMLDeclaration();
            BaseListXMLPart baseListXMLPart = new BaseListXMLPart(xmlWriter);
            DataAccess dataAccess = sctServer.getScrutariSession().getScrutariDB().openDataAccess();
            baseListXMLPart.appendBaseList(sctServer, dataAccess);
            dataAccess.close();
        }

    }


    private static class LogListXmlProducer implements XmlProducer {

        private SctServer sctServer;

        private LogListXmlProducer(SctServer sctServer) {
            this.sctServer = sctServer;
        }

        @Override
        public void writeXml(Appendable appendable) throws IOException {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(appendable);
            xmlWriter.appendXMLDeclaration();
            LogListXMLPart logXMLPart = new LogListXMLPart(xmlWriter);
            logXMLPart.appendLogList(sctServer.getSctServerStorage().getLogStorage());
        }

    }

}
