/*Scrutari - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.explore.xml;

import fr.exemole.sctserver.api.ScrutariSourceManager;
import fr.exemole.sctserver.api.SctServer;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import java.io.IOException;
import java.net.URL;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.data.BaseData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.db.api.DataAccess;


/**
 *
 * @author Vincent Calame
 */
public class BaseListXMLPart extends XMLPart {

    public BaseListXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void appendBaseList(SctServer sctServer, DataAccess dataAccess) throws IOException {
        ScrutariSourceManager scrutariSourceManager = sctServer.getScrutariInfoManager();
        int baseCount = dataAccess.getBaseCount();
        openTag("base-list");
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            ScrutariSource scrutariSource = scrutariSourceManager.getScrutariSouce(baseData.getScrutariSourceName());
            ScrutariSourceDef scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            BaseURI baseURI = baseData.getBaseURI();
            startOpenTag("base");
            addAttribute("uri", baseURI.toString());
            endOpenTag();
            addSimpleElement("source-name", scrutariSourceDef.getName());
            addSimpleElement("info-url", getUrlString(scrutariSourceDef.getUrl()));
            addSimpleElement("last-update", DateInteger.toISOString(scrutariSource.getLastUpdate()));
            addSimpleElement("authority-uuid", baseURI.getAuthorityUUID());
            addSimpleElement("base-name", baseURI.getBaseName());
            int langCount = baseData.getLangDataCount();
            openTag("intitule-short");
            for (int j = 0; j < langCount; j++) {
                BaseData.BaseLangData baseLangData = baseData.getLangDataByIndex(j);
                Lang lang = baseLangData.getLang();
                String lib = baseLangData.getLibelleString(BaseData.INTITULE_SHORT);
                if (lib != null) {
                    XMLUtils.addLibElement(this, lang, lib);
                }
            }
            closeTag("intitule-short");
            openTag("intitule-long");
            for (int j = 0; j < langCount; j++) {
                BaseData.BaseLangData baseLangData = baseData.getLangDataByIndex(j);
                Lang lang = baseLangData.getLang();
                String lib = baseLangData.getLibelleString(BaseData.INTITULE_LONG);
                if (lib != null) {
                    XMLUtils.addLibElement(this, lang, lib);
                }
            }
            closeTag("intitule-long");
            closeTag("base");
        }
        closeTag("base-list");
    }

    private static String getUrlString(URL url) {
        if (url == null) {
            return null;
        }
        return url.toString();
    }

}
