/* Scrutari - Copyright (c) 2012-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.export.contentstreams;

import fr.exemole.sctserver.api.ScrutariSession;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdXML;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.OdZipEngine;
import net.mapeadores.opendocument.io.OdSource;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.mimetype.MimeTypeConstants;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.data.BaseData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.db.tools.util.DataLibelleUtils;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.tools.options.OptionsUtils;


/**
 *
 * @author Vincent Calame
 */
public class ResultOdsStream implements ResponseHandler {

    private final ScrutariSession scrutariSession;
    private final FicheSearchResult ficheSearchResult;
    private final Lang langUi;

    public ResultOdsStream(ScrutariSession scrutariSession, FicheSearchResult ficheSearchResult, Lang langUi) {
        this.scrutariSession = scrutariSession;
        this.ficheSearchResult = ficheSearchResult;
        this.langUi = langUi;
    }

    @Override
    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(MimeTypeConstants.ODS);
        OutputStream outputStream = response.getOutputStream();
        OdZip odZip = OdZip.newInstance(OdUtils.OD_SPREADSHEET).setContentOdSource(new ContentOdSource());
        OdZipEngine.run(outputStream, odZip);
        outputStream.close();
    }


    private class ContentOdSource implements OdSource {

        private ContentOdSource() {
        }

        @Override
        public void writeStream(OutputStream outputStream) throws IOException {
            BufferedWriter buf = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(buf);
            xmlWriter.appendXMLDeclaration();
            ResultXMLPart resultXMLPart = new ResultXMLPart(xmlWriter);
            resultXMLPart.start();
            resultXMLPart.addResult();
            resultXMLPart.end();
            buf.flush();
        }


        private class ResultXMLPart extends XMLPart {

            private final ScrutariDB scrutariDB;
            private final DataAccess dataAccess;
            private final GlobalSearchOptions globalSearchOptions;
            private final Map<Integer, BaseInfo> baseInfoMap = new HashMap<Integer, BaseInfo>();
            private final Map<Integer, String> motcleLibMap = new HashMap<Integer, String>();

            private ResultXMLPart(XMLWriter xmlWriter) {
                super(xmlWriter);
                this.scrutariDB = scrutariSession.getScrutariDB();
                this.dataAccess = scrutariDB.openDataAccess();
                this.globalSearchOptions = scrutariSession.getGlobalSearchOptions();
            }

            private void start() throws IOException {
                OdXML.openDocumentContent(this);
                OdXML.openBody(this);
                OdXML.openSpreadsheet(this);
            }

            private void end() throws IOException {
                OdXML.closeSpreadsheet(this);
                OdXML.closeBody(this);
                OdXML.closeDocumentContent(this);
                dataAccess.close();
            }

            private void addResult() throws IOException {
                int groupCount = ficheSearchResult.getFicheSearchResultGroupCount();
                for (int i = 0; i < groupCount; i++) {
                    FicheSearchResultGroup ficheSearchResultGroup = ficheSearchResult.getFicheSearchResultGroup(i);
                    int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
                    String tableName;
                    Category category = globalSearchOptions.getCategoryByRank(ficheSearchResultGroup.getCategoryRank());
                    if (category != null) {
                        CategoryDef categoryDef = category.getCategoryDef();
                        tableName = OptionsUtils.checkLibelleString(categoryDef, langUi, categoryDef.getName());
                    } else {
                        if (groupCount == 1) {
                            tableName = "Result";
                        } else {
                            int pos = i + 1;
                            tableName = "Result " + pos;
                        }
                    }
                    OdXML.openTable(this, tableName);
                    for (int j = 0; j < ficheByGroupCount; j++) {
                        FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                        int codefiche = ficheSearchResultInfo.getCode();
                        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                        addFicheData(ficheData);
                    }
                    OdXML.closeTable(this);

                }
            }

            private void addFicheData(FicheData ficheData) throws IOException {
                int codefiche = ficheData.getCodeFiche();
                BaseInfo baseInfo = getBaseInfo(ficheData.getCodeBase());
                OdXML.openTableRow(this);
                OdXML.addStringTableCell(this, ficheData.getTitre());
                OdXML.addStringTableCell(this, ficheData.getSoustitre());
                OdXML.addNumberTableCell(this, DateInteger.getAnnee(ficheData.getDateInteger()));
                OdXML.addStringTableCell(this, baseInfo.libelle);
                OdXML.addStringTableCell(this, ficheData.getLang().toString());
                OdXML.addLinkStringTableCell(this, ficheData.getHref(baseInfo.baseCheck.getCheckedLangUi()));
                int max = ficheData.getComplementMaxNumber();
                StringBuilder motcleBuilder = new StringBuilder();
                CodeList thesaurusCodeList = dataAccess.getThesaurusCodeList();
                int thesaurusCount = thesaurusCodeList.getCodeCount();
                for (int i = 0; i < thesaurusCount; i++) {
                    IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codefiche, thesaurusCodeList.getCode(i));
                    int indexationCount = indexationCodeList.getCodeCount();
                    for (int j = 0; j < indexationCount; j++) {
                        int codemotcle = indexationCodeList.getCode(j);
                        String libelleString = getMotcleLib(codemotcle);
                        if (libelleString.length() > 0) {
                            if (motcleBuilder.length() > 0) {
                                motcleBuilder.append("; ");
                            }
                            motcleBuilder.append(libelleString);
                        }
                    }
                }
                OdXML.addStringTableCell(this, motcleBuilder.toString());
                for (int i = 1; i <= max; i++) {
                    OdXML.addStringTableCell(this, ficheData.getComplementByNumber(i));
                }
                OdXML.closeTableRow(this);
            }

            private String getMotcleLib(int codemotcle) {
                String libelleString = motcleLibMap.get(codemotcle);
                if (libelleString != null) {
                    return libelleString;
                }
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                Libelle libelle = motcleData.getLibelleByLang(langUi);
                if (libelle == null) {
                    if (motcleData.getLibelleCount() > 0) {
                        libelle = motcleData.getLibelleByIndex(0);
                    }
                }
                if (libelle != null) {
                    libelleString = libelle.getLibelleString();
                } else {
                    libelleString = "";
                }
                motcleLibMap.put(codemotcle, libelleString);
                return libelleString;
            }

            private BaseInfo getBaseInfo(int codebase) {
                BaseInfo baseCheck = baseInfoMap.get(codebase);
                if (baseCheck != null) {
                    return baseCheck;
                }
                BaseData baseData = dataAccess.getBaseDataByCode(codebase);
                baseCheck = new BaseInfo(baseData);
                baseInfoMap.put(codebase, baseCheck);
                return baseCheck;
            }


            private class BaseInfo {

                private final BaseData baseData;
                private final BaseCheck baseCheck;
                private final String libelle;

                private BaseInfo(BaseData baseData) {
                    this.baseData = baseData;
                    this.baseCheck = BaseCheck.check(baseData, langUi);
                    this.libelle = DataLibelleUtils.checkLibelleString(baseData, BaseData.INTITULE_SHORT, baseCheck.getCheckedLangUi(), baseData.getBaseURI().getBaseName());
                }

            }

        }

    }

}
