/* ScrutariLib_DB - Copyright (c) 2007-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.util;

import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;


/**
 *
 * @author Vincent Calame
 */
public final class DataLibelleUtils {

    private DataLibelleUtils() {
    }

    public static String checkLibelleString(CorpusData corpusData, int fieldNumber, Lang lang, String defaultString) {
        CorpusData.CorpusLangData corpusLangData = corpusData.getLangDataByLang(lang);
        String libelleString = getLibelleString(corpusLangData, fieldNumber);
        if (libelleString != null) {
            return libelleString;
        }
        if (!lang.isRootLang()) {
            libelleString = getLibelleString(corpusData.getLangDataByLang(lang.getRootLang()), fieldNumber);
            if (libelleString != null) {
                return libelleString;
            }
        }
        int count = corpusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            corpusLangData = corpusData.getLangDataByIndex(i);
            libelleString = getLibelleString(corpusLangData, fieldNumber);
            if (libelleString != null) {
                return libelleString;
            }
        }
        return defaultString;
    }

    public static String checkLibelleString(BaseData baseData, short type, Lang lang, String defaultString) {
        BaseData.BaseLangData baseLangData = baseData.getLangDataByLang(lang);
        String libelleString = getLibelleString(baseLangData, type);
        if (libelleString != null) {
            return libelleString;
        }
        if (!lang.isRootLang()) {
            libelleString = getLibelleString(baseData.getLangDataByLang(lang.getRootLang()), type);
            if (libelleString != null) {
                return libelleString;
            }
        }
        int count = baseData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            baseLangData = baseData.getLangDataByIndex(i);
            libelleString = getLibelleString(baseLangData, type);
            if (libelleString != null) {
                return libelleString;
            }
        }
        return defaultString;
    }

    public static String checkLibelleString(ThesaurusData thesaurusData, Lang lang, String defaultString) {
        ThesaurusData.ThesaurusLangData thesaurusLangData = thesaurusData.getLangDataByLang(lang);
        String res = getLibelleString(thesaurusLangData);
        if (res != null) {
            return res;
        }
        if (!lang.isRootLang()) {
            res = getLibelleString(thesaurusData.getLangDataByLang(lang.getRootLang()));
            if (res != null) {
                return res;
            }
        }
        int count = thesaurusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            thesaurusLangData = thesaurusData.getLangDataByIndex(i);
            res = getLibelleString(thesaurusLangData);
            if (res != null) {
                return res;
            }
        }
        return defaultString;
    }

    private static String getLibelleString(CorpusData.CorpusLangData corpusLangData, int fieldNumber) {
        if (corpusLangData == null) {
            return null;
        }
        Libelle lib = corpusLangData.getLibelle(fieldNumber);
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    private static String getLibelleString(BaseData.BaseLangData baseLangData, short type) {
        if (baseLangData == null) {
            return null;
        }
        Libelle lib = baseLangData.getLibelle(type);
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    private static String getLibelleString(ThesaurusData.ThesaurusLangData thesaurusLangData) {
        if (thesaurusLangData == null) {
            return null;
        }
        Libelle lib = thesaurusLangData.getLibelle();
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

}
