/* ScrutariLib_DB - Copyright (c) 2005-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.util;

import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.db.api.stats.ScrutariDBStats;


/**
 * Propose des méthodes statitiques utilitaires pour récupérer des Datarmations
 * sur les bases.
 *
 * @author Vincent Calame
 */
public class DataUtils {

    static final char SEUIL1 = '\u0080';
    static final char SEUIL2 = '\u0800';

    /**
     * Retourne le nombre de caractères supplémentaires introduit par le codage
     * UTF8 du caractère en argument. Par exemple, la méthode retournera 0 pour
     * le caractère « a » qui en UTF-8 s'écrit « a » et elle retournera 1 pour
     * le caractère « é » qui en UTF-8 s'écrit « Ã© ».
     */
    public static int getDecalageUTF8(char c) {
        if (c < SEUIL1) {
            return 0;
        }
        if (c < SEUIL2) {
            return 1;
        }
        return 2;
    }

    private DataUtils() {
    }

    public static String getHref(FicheData ficheData, DataAccess dataAccess, Lang langUi) {
        BaseData baseData = dataAccess.getBaseDataByCode(dataAccess.getCorpusDataByCode(ficheData.getCodeCorpus()).getCodeBase());
        Lang checkedLang = baseData.checkLangUi(langUi);
        return ficheData.getHref(checkedLang);
    }

    public static int getFicheCountByCorpus(DataAccess dataAccess, CodeList corpusCodeList, Lang[] langArray, ScrutariDBStats scrutariDBStats) {
        int resultat = 0;
        if (langArray != null) {
            if (corpusCodeList == null) {
                for (int i = 0; i < dataAccess.getCorpusCount(); i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                    int codecorpus = corpusData.getCodeCorpus();
                    FicheStats ficheStats = scrutariDBStats.getFicheStats(codecorpus);
                    int langCount = ficheStats.getLangCount();
                    for (int k = 0; k < langCount; k++) {
                        Lang lang = ficheStats.getLang(k);
                        if (contains(langArray, lang)) {
                            resultat = resultat + ficheStats.getFicheCount(k);
                        }
                    }
                }
            } else {
                int corpusCount = corpusCodeList.getCodeCount();
                for (int i = 0; i < corpusCount; i++) {
                    int codecorpus = corpusCodeList.getCode(i);
                    FicheStats ficheStats = scrutariDBStats.getFicheStats(codecorpus);
                    int langCount = ficheStats.getLangCount();
                    for (int k = 0; k < langCount; k++) {
                        Lang lang = ficheStats.getLang(k);
                        if (contains(langArray, lang)) {
                            resultat = resultat + ficheStats.getFicheCount(k);
                        }
                    }
                }
            }
        } else {
            if (corpusCodeList == null) {
                for (int i = 0; i < dataAccess.getCorpusCount(); i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                    resultat = resultat + corpusData.getFicheCount();
                }
            } else {
                int corpusCount = corpusCodeList.getCodeCount();
                for (int i = 0; i < corpusCount; i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(corpusCodeList.getCode(i));
                    if (corpusData != null) {
                        resultat = resultat + corpusData.getFicheCount();
                    }
                }
            }
        }
        return resultat;
    }

    public static int getFicheCountByCorpus(DataAccess dataAccess, int[] codecorpusArray, Lang[] langArray, ScrutariDBStats scrutariDBStats) {
        int resultat = 0;
        if (langArray != null) {
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                FicheStats ficheStats = scrutariDBStats.getFicheStats(codecorpus);
                int langCount = ficheStats.getLangCount();
                for (int k = 0; k < langCount; k++) {
                    Lang lang = ficheStats.getLang(k);
                    if (contains(langArray, lang)) {
                        resultat = resultat + ficheStats.getFicheCount(k);
                    }
                }
            }
        } else {
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpusArray[i]);
                if (corpusData != null) {
                    resultat = resultat + corpusData.getFicheCount();
                }
            }
        }
        return resultat;
    }

    public static int getFicheCountByBase(DataAccess dataAccess, CodeList baseCodeList, Lang[] langArray, ScrutariDBStats scrutariDBStats) {
        int resultat = 0;
        if (langArray != null) {
            if (baseCodeList == null) {
                for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                    BaseData baseData = dataAccess.getBaseDataByIndex(i);
                    for (int j = 0; j < baseData.getCorpusCount(); j++) {
                        int codecorpus = baseData.getCodeCorpus(j);
                        FicheStats ficheStats = scrutariDBStats.getFicheStats(codecorpus);
                        int langCount = ficheStats.getLangCount();
                        for (int k = 0; k < langCount; k++) {
                            Lang lang = ficheStats.getLang(k);
                            if (contains(langArray, lang)) {
                                resultat = resultat + ficheStats.getFicheCount(k);
                            }
                        }
                    }
                }
            } else {
                int baseCount = baseCodeList.getCodeCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseData baseData = dataAccess.getBaseDataByCode(baseCodeList.getCode(i));
                    if (baseData != null) {
                        for (int j = 0; j < baseData.getCorpusCount(); j++) {
                            int codecorpus = baseData.getCodeCorpus(j);
                            FicheStats ficheStats = scrutariDBStats.getFicheStats(codecorpus);
                            int langCount = ficheStats.getLangCount();
                            for (int k = 0; k < langCount; k++) {
                                Lang lang = ficheStats.getLang(k);
                                if (contains(langArray, lang)) {
                                    resultat = resultat + ficheStats.getFicheCount(k);
                                }
                            }
                        }
                    }
                }
            }
        } else {
            if (baseCodeList == null) {
                for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                    BaseData baseData = dataAccess.getBaseDataByIndex(i);
                    for (int j = 0; j < baseData.getCorpusCount(); j++) {
                        CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(j));
                        resultat = resultat + corpusData.getFicheCount();
                    }
                }
            } else {
                int baseCount = baseCodeList.getCodeCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseData basData = dataAccess.getBaseDataByCode(baseCodeList.getCode(i));
                    if (basData != null) {
                        for (int j = 0; j < basData.getCorpusCount(); j++) {
                            CorpusData corpusData = dataAccess.getCorpusDataByCode(basData.getCodeCorpus(j));
                            resultat = resultat + corpusData.getFicheCount();
                        }
                    }
                }
            }
        }
        return resultat;
    }

    public static int getMotcleCountByBase(DataAccess dataAccess, CodeList baseCodeList) {
        int resultat = 0;
        if (baseCodeList == null) {
            for (int i = 0; i < dataAccess.getBaseCount(); i++) {
                BaseData baseData = dataAccess.getBaseDataByIndex(i);
                for (int j = 0; j < baseData.getThesaurusCount(); j++) {
                    ThesaurusData corpusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(j));
                    resultat = resultat + corpusData.getMotcleCount();
                }
            }
        } else {
            int baseCount = baseCodeList.getCodeCount();
            for (int i = 0; i < baseCount; i++) {
                BaseData basData = dataAccess.getBaseDataByCode(baseCodeList.getCode(i));
                if (basData != null) {
                    for (int j = 0; j < basData.getCorpusCount(); j++) {
                        ThesaurusData corpusData = dataAccess.getThesaurusDataByCode(basData.getCodeThesaurus(j));
                        resultat = resultat + corpusData.getMotcleCount();
                    }
                }
            }
        }
        return resultat;
    }

    private static boolean contains(Lang[] array, Lang value) {
        int length = array.length;
        for (int i = 0; i < length; i++) {
            if (array[i].equals(value)) {
                return true;
            }
        }
        return false;
    }

    private static boolean contains(LangList langList, Lang value) {
        int langCount = langList.getLangCount();
        for (int i = 0; i < langCount; i++) {
            if (langList.getLang(i).equals(value)) {
                return true;
            }
        }
        return false;
    }

}
