/* ScrutariLib_Lexie - Copyright (c) 2005-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.lexie;

import java.io.IOException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.lexie.LexieUnit;


/**
 * Point particulier des unités lexicales : elles correspondent à une langue donnée.
 *
 * @author Vincent Calame
 */
public class ScrutariLexieUnit implements LexieUnit {

    private final long lexieId;
    private final Lang lang;
    private final String collatedLexie;
    private String canonicalLexie;
    private boolean hasFicheLink = false;
    private boolean hasMotcleLink = false;

    public ScrutariLexieUnit(long lexieId, Lang lang, String collatedLexie) {
        this.lexieId = lexieId;
        this.lang = lang;
        this.collatedLexie = collatedLexie;
    }

    public Lang getLang() {
        return lang;
    }

    @Override
    public String getCollatedLexie() {
        return collatedLexie;

    }

    public long getLexieId() {
        return lexieId;
    }

    @Override
    public String getCanonicalLexie() {
        return canonicalLexie;
    }

    public void setCanonicalLexie(String canonicalLexie) {
        this.canonicalLexie = canonicalLexie;
    }

    public boolean hasMotcleAndLexieLink() {
        return hasMotcleLink;
    }

    public boolean hasFicheAndLexieLink() {
        return hasFicheLink;
    }

    public void setHasFicheLink(boolean hasFicheLink) {
        this.hasFicheLink = hasFicheLink;
    }

    public void setHasMotcleLink(boolean hasMotcleLink) {
        this.hasMotcleLink = hasMotcleLink;
    }

    public static ScrutariLexieUnit fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        long lexieId = primitivesReader.readLong();
        Lang lang = Lang.build(primitivesReader.readString());
        String collatedLexie = primitivesReader.readString();
        ScrutariLexieUnit scrutariLexieUnit = new ScrutariLexieUnit(lexieId, lang, collatedLexie);
        scrutariLexieUnit.setCanonicalLexie(primitivesReader.readString());
        scrutariLexieUnit.setHasFicheLink(primitivesReader.readBoolean());
        scrutariLexieUnit.setHasMotcleLink(primitivesReader.readBoolean());
        return scrutariLexieUnit;
    }

    public static void toPrimitives(ScrutariLexieUnit scrutariLexieUnit, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeLong(scrutariLexieUnit.lexieId);
        primitivesWriter.writeString(scrutariLexieUnit.lang.toString());
        primitivesWriter.writeString(scrutariLexieUnit.getCollatedLexie());
        primitivesWriter.writeString(scrutariLexieUnit.getCanonicalLexie());
        primitivesWriter.writeBoolean(scrutariLexieUnit.hasFicheLink);
        primitivesWriter.writeBoolean(scrutariLexieUnit.hasMotcleLink);
    }

}
