/* ScrutariLib_SearchEngine - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.DefaultLibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.searchengine.api.options.CategoryDef;


/**
 *
 * @author Vincent Calame
 */
public class CategoryDefBuilder {

    private final String name;
    private final InternalLibelleHolder libelleHolder;
    private final List<CorpusURI> uriList = new ArrayList<CorpusURI>();
    private int rank;

    public CategoryDefBuilder(String name) {
        this.name = name;
        this.libelleHolder = new InternalLibelleHolder();
    }

    public void addLibelle(Libelle libelle) {
        libelleHolder.internalSetLibelle(libelle);
    }

    public void addCorpusURI(CorpusURI corpusURI) {
        uriList.add(corpusURI);
    }

    public void setRank(int rank) {
        this.rank = rank;
    }

    public CategoryDef toCategoryDef() {
        CorpusURI[] corpusURIArray = uriList.toArray(new CorpusURI[uriList.size()]);
        return new InternalCategoryDef(name, rank, libelleHolder, corpusURIArray);
    }


    private static class InternalCategoryDef implements CategoryDef {

        private final String name;
        private final int rank;
        private final CorpusURI[] corpusURIArray;
        private final LibelleHolder libelleHolder;


        private InternalCategoryDef(String name, int rank, LibelleHolder libelleHolder, CorpusURI[] corpusURIArray) {
            this.name = name;
            this.rank = rank;
            this.corpusURIArray = corpusURIArray;
            this.libelleHolder = libelleHolder;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public int getCorpusURICount() {
            return corpusURIArray.length;
        }

        @Override
        public CorpusURI getCorpusURI(int i) {
            return corpusURIArray[i];
        }

        @Override
        public int getRank() {
            return rank;
        }

        @Override
        public int getLibelleCount() {
            return libelleHolder.getLibelleCount();
        }

        @Override
        public Libelle getLibelleByIndex(int i) {
            return libelleHolder.getLibelleByIndex(i);
        }

        @Override
        public Libelle getLibelleByLang(Lang lang) {
            return libelleHolder.getLibelleByLang(lang);
        }

    }


    private static class InternalLibelleHolder extends DefaultLibelleHolder {

        private InternalLibelleHolder() {
            super();
        }

        private void internalSetLibelle(Libelle libelle) {
            innerPutLibelle(libelle);
        }

    }
}
