/* ScrutariLib_SearchEngine - Copyright (c) 2011-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchResultInfoBuilder {

    private final static int[] EMPTY_SCOREARRAY = {0, 0, 0};
    private InternalFicheSearchResultInfo ficheSearchResultInfo = new InternalFicheSearchResultInfo();

    public FicheSearchResultInfoBuilder() {
    }

    public void setCode(int code) {
        ficheSearchResultInfo.code = code;
    }

    public void setMotcleCodeArray(int[] motcleCodeArray) {
        ficheSearchResultInfo.motcleCodeArray = motcleCodeArray;
    }

    public void setInAlineaOccurrenceListArray(InAlineaOccurrenceList[] inAlineaOccurrenceListArray) {
        ficheSearchResultInfo.inAlineaOccurrenceListArray = inAlineaOccurrenceListArray;
    }

    public void setScoreArray(int[] scoreArray) {
        ficheSearchResultInfo.scoreArray = scoreArray;
    }

    public FicheSearchResultInfo toFicheSearchResultInfo() {
        return ficheSearchResultInfo;
    }


    private static class InternalFicheSearchResultInfo implements FicheSearchResultInfo {

        private int code;
        private int[] motcleCodeArray = null;
        private InAlineaOccurrenceList[] inAlineaOccurrenceListArray;
        private int[] scoreArray = EMPTY_SCOREARRAY;

        private InternalFicheSearchResultInfo() {
        }

        public int getCode() {
            return code;
        }

        public int getMotcleCount() {
            if (motcleCodeArray == null) {
                return 0;
            }
            return motcleCodeArray.length;
        }

        public int getMotcleCode(int i) {
            return motcleCodeArray[i];
        }

        public int getInAlineaOccurrenceListCount() {
            if (inAlineaOccurrenceListArray == null) {
                return 0;
            }
            return inAlineaOccurrenceListArray.length;
        }

        public InAlineaOccurrenceList getInAlineaOccurrenceList(int i) {
            return inAlineaOccurrenceListArray[i];
        }

        public InAlineaOccurrenceList getInAlineaOccurrenceListByAlinea(int alineaInteger) {
            if (inAlineaOccurrenceListArray == null) {
                return null;
            }
            int count = inAlineaOccurrenceListArray.length;
            for (int i = 0; i < count; i++) {
                InAlineaOccurrenceList inAlineaOccurrenceList = inAlineaOccurrenceListArray[i];
                if (inAlineaOccurrenceList.getAlineaInteger() == alineaInteger) {
                    return inAlineaOccurrenceList;
                }
            }
            return null;
        }

        public int[] getScoreArray() {
            return scoreArray;
        }

    }

}
