/* ScrutariLib_Supermotcle - Copyright (c) 2013-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.supermotcle.tools;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Libelle;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.supermotcle.api.SupermotcleHolder;
import net.scrutari.supermotcle.api.SupermotcleProvider;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleProviderBuilder {

    private final Map<Lang, SupermotcleHolderBuilder> builderMap = new HashMap<Lang, SupermotcleHolderBuilder>();
    private final Map<Lang, SupermotcleHolder> supermotcleHolderMap = new HashMap<Lang, SupermotcleHolder>();

    private SupermotcleProviderBuilder() {
    }

    private SupermotcleProvider toSupermotcleProvider() {
        return new InternalSupermotcleProvider(supermotcleHolderMap);
    }

    private void build(DataAccess dataAccess) {
        int thesaurusCount = dataAccess.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                initMotcle(dataAccess, thesaurusData, motcleData);
            }
        }
        for (Map.Entry<Lang, SupermotcleHolderBuilder> entry : builderMap.entrySet()) {
            Lang lang = entry.getKey();
            SupermotcleHolderBuilder builder = entry.getValue();
            SupermotcleHolder supermotcleHolder = builder.toSupermotcleMap();
            supermotcleHolderMap.put(lang, supermotcleHolder);
        }
    }

    private void initMotcle(DataAccess dataAccess, ThesaurusData thesaurusData, MotcleData motcleData) {
        int codemotcle = motcleData.getCodeMotcle();
        int libelleCount = motcleData.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            SupermotcleHolderBuilder supermotcleHolderBuilder = getOrCreateSupermotcleHolderBuilder(dataAccess, libelle.getLang());
            supermotcleHolderBuilder.addMotcle(thesaurusData.getCodeBase(), thesaurusData.getCodeThesaurus(), codemotcle, libelle.getLibelleString());
        }
    }

    private SupermotcleHolderBuilder getOrCreateSupermotcleHolderBuilder(DataAccess dataAccess, Lang lang) {
        SupermotcleHolderBuilder supermotcleHolderBuilder = builderMap.get(lang);
        if (supermotcleHolderBuilder == null) {
            supermotcleHolderBuilder = new SupermotcleHolderBuilder(dataAccess, lang);
            builderMap.put(lang, supermotcleHolderBuilder);
        }
        return supermotcleHolderBuilder;
    }

    public static SupermotcleProvider build(ScrutariDB scrutariDB) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SupermotcleProviderBuilder builder = new SupermotcleProviderBuilder();
        builder.build(dataAccess);
        dataAccess.close();
        return builder.toSupermotcleProvider();
    }


    private static class InternalSupermotcleProvider implements SupermotcleProvider {

        private final Map<Lang, SupermotcleHolder> supermotcleHolderMap;

        private InternalSupermotcleProvider(Map<Lang, SupermotcleHolder> supermotcleHolderMap) {
            this.supermotcleHolderMap = supermotcleHolderMap;
        }

        @Override
        public SupermotcleHolder getSupermotcleHolder(Lang lang) {
            return supermotcleHolderMap.get(lang);
        }

    }

}
