/* SctServer - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.api.storage;

import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.feed.SctFeedCache;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Set;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.caches.DataCacheReaderFactory;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.db.api.caches.LexieCacheWriter;


/**
 *
 * @author Vincent Calame
 */
public interface SctServerStorage {

    public final static String INFOS_NAME = "infos";
    public final static String STATUS_NAME = "status";

    /**
     * Peut retourner null.
     */
    public InputStream getScrutariData(String scrutariSourceName);

    public String getContent(String contentName);

    public void saveContent(String contentName, String content);

    public OutputStream getScrutariDataOutputStream(String scrutariSourceName);

    public ConfDirectory getConfDirectory();

    public void saveURITree(URITree uriTree, ScrutariDBName scrutariDBName);

    /**
     * Si null, renvoie l'arbre complet.
     */
    public URITree getURITree(ScrutariDBName scrutariDBName);

    /**
     * Dates de la plus récente à la plus ancienne
     */
    public ScrutariDBName[] getAddScrutariDBNameArray();

    public QIdStorage getQIdStorage();

    public LogStorage getLogStorage();

    public DataCacheWriter getDataCacheWriter(ScrutariDBName scrutariDbName);

    public LexieCacheWriter getLexieCacheWriter(ScrutariDBName scrutariDbName);

    public SctFeedCache getSctFeedCache(ScrutariDBName scrutariDbName);

    public FicheSearchCache getFicheSearchCache(ScrutariDBName scrutariDbName);

    public void cleanCache(Set<ScrutariDBName> remainingSet);

    public void cleanTransientCache(ScrutariDBName scrutariDbName);

    /**
     *
     * @param currentVersion peut être nul
     */
    public CacheCheck checkLastCache(ScrutariDBName currentName);


    public interface CacheCheck {

        public ScrutariDBName getScrutariDBName();

        public DataCacheReaderFactory getDataCacheReaderFactory();

        public LexieCacheReaderFactory getLexieCacheReaderFactory();

    }

}
