/* SctServer - Copyright (c) 2010-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.BaseDataJson;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.BaseData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;


/**
 *
 * @author Vincent Calame
 */
public class BaseJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final Lang langUi;

    public BaseJsonProducer(int version, ScrutariDB scrutariDB, Lang langUi) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.langUi = langUi;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            if (version > 1) {
                printVersion2(jsonWriter, dataAccess);
            } else {
                printVersion1(jsonWriter, dataAccess);
            }
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

    private void printVersion2(JSONWriter jsonWriter, DataAccess dataAccess) throws JSONException {
        int baseCount = dataAccess.getBaseCount();
        jsonWriter.key("baseArray");
        jsonWriter.array();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            BaseDataJson.object(jsonWriter, baseData, langUi, scrutariDB.getStats());
        }
        jsonWriter.endArray();
    }

    private void printVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws JSONException {
        jsonWriter.key("baseData");
        jsonWriter.object();
        int baseCount = dataAccess.getBaseCount();
        jsonWriter.key("baseCount");
        jsonWriter.value(baseCount);
        if (baseCount > 0) {
            jsonWriter.key("baseArray");
            jsonWriter.array();
            for (int i = 0; i < baseCount; i++) {
                BaseData baseData = dataAccess.getBaseDataByIndex(i);
                BaseDataJson.object(jsonWriter, baseData, langUi, scrutariDB.getStats());
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

}
