/* SctServer  - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.request.CountFilter;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.util.DataUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheCountJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final CountFilter countFilter;

    public FicheCountJsonProducer(int version, ScrutariDB scrutariDB, CountFilter countFilter) {
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.countFilter = countFilter;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("ficheCount");
            jsonWriter.value(getFicheCount(dataAccess));
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

    private int getFicheCount(DataAccess dataAccess) {
        CodeList corpusCodeList = countFilter.getCorpusCodeList();
        if (corpusCodeList != null) {
            return DataUtils.getFicheCountByCorpus(dataAccess, corpusCodeList, countFilter.getLangArray(), scrutariDB.getStats());
        }
        return DataUtils.getFicheCountByBase(dataAccess, countFilter.getBaseCodeList(), countFilter.getLangArray(), scrutariDB.getStats());
    }

}
