/* SctServer - Copyright (c) 2012-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.MotcleDataJson;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.CodeList;


/**
 *
 * @author Vincent Calame
 */
public class MotcleJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final CodeList motcleCodeList;
    private final MotcleFieldsFilter motcleFieldsFilter;

    public MotcleJsonProducer(int version, ScrutariDB scrutariDB, CodeList codemotcleList, MotcleFieldsFilter motcleFieldsFilter) {
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.motcleCodeList = codemotcleList;
        this.motcleFieldsFilter = motcleFieldsFilter;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            if (version > 1) {
                printVersion2(jsonWriter, dataAccess);
            } else {
                printVersion1(jsonWriter, dataAccess);
            }
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

    private void printVersion2(JSONWriter jsonWriter, DataAccess dataAccess) throws JSONException {
        int motcleCount = motcleCodeList.getCodeCount();
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (int i = 0; i < motcleCount; i++) {
            int codemotcle = motcleCodeList.getCode(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
            jsonWriter.object();
            MotcleDataJson.object(jsonWriter, motcleData, motcleFieldsFilter, null, null);
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

    private void printVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws JSONException {
        jsonWriter.key("motcleData");
        jsonWriter.object();
        int motcleCount = motcleCodeList.getCodeCount();
        jsonWriter.key("motcleCount");
        jsonWriter.value(motcleCount);
        if (motcleCount > 0) {
            jsonWriter.key("motcleArray");
            jsonWriter.array();
            for (int i = 0; i < motcleCount; i++) {
                int codemotcle = motcleCodeList.getCode(i);
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                jsonWriter.object();
                MotcleDataJson.object(jsonWriter, motcleData, motcleFieldsFilter, null, null);
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

}
