/* SctServer - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.CollectErrorHandler;
import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.cert.X509Certificate;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public final class CollectUtils {

    private CollectUtils() {
    }

    public static void initSslFactory() {
        try {
            SSLContext sc = SSLContext.getInstance("SSLv3");
            TrustManager[] tma = {new InternalTrustManager()};
            sc.init(null, tma, null);
            SSLSocketFactory ssf = sc.getSocketFactory();
            HttpsURLConnection.setDefaultSSLSocketFactory(ssf);
        } catch (Exception e) {
            throw new NestedLibraryException(e);
        }
    }

    public static boolean hasChange(ScrutariSource scrutariSource, CollectResult collectResult) {
        short newState = collectResult.getState();
        int newDate = collectResult.getLastUpdateDateInteger();
        short currentState = scrutariSource.getState();
        if (currentState != ScrutariSource.OK_STATE) {
            if (newState == ScrutariSource.OK_STATE) {
                return true;
            } else {
                return false;
            }
        }
        if (newState != ScrutariSource.OK_STATE) {
            return true;
        }
        if (newDate != scrutariSource.getLastUpdate()) {
            return true;
        } else {
            return false;
        }
    }

    public static CollectResult toCollectResult(short state, int lastUpdate) {
        return new InternalCollectResult(state, lastUpdate);
    }

    public static boolean testConnection(URLConnection urlConnection, CollectErrorHandler collectErrorHandler) throws IOException {
        URL url = urlConnection.getURL();
        if (url.getProtocol().equals("https")) {
            HttpsURLConnection httpsCon = (HttpsURLConnection) urlConnection;
            int reponse = httpsCon.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                collectErrorHandler.warningDownload(url, httpsCon.getResponseMessage());
                return false;
            } else {
                return true;
            }
        } else {
            HttpURLConnection con = (HttpURLConnection) urlConnection;
            int reponse = con.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                collectErrorHandler.warningDownload(url, con.getResponseMessage());
                return false;
            } else {
                return true;
            }
        }
    }

    public static String stateToString(short state) {
        switch (state) {
            case ScrutariSource.OK_STATE:
                return "state_ok";
            case ScrutariSource.UNKNOWN_STATE:
                return "state_unknown";
            case ScrutariSource.INFO_NOCONNECTION_STATE:
                return "state_info_noconnection";
            case ScrutariSource.INFO_BROKENURL_STATE:
                return "state_info_brokenurl";
            case ScrutariSource.INFO_DOWNLOADERROR_STATE:
                return "state_info_downloaderror";
            case ScrutariSource.INFO_NOTXML_STATE:
                return "state_info_notxml";
            case ScrutariSource.INFO_WRONGXML_STATE:
                return "state_info_wrongxml";
            case ScrutariSource.DATA_NOCONNECTION_STATE:
                return "state_data_noconnection";
            case ScrutariSource.DATA_BROKENURL_STATE:
                return "state_data_brokenurl";
            case ScrutariSource.DATA_DOWNLOADERROR_STATE:
                return "state_data_downloaderror";
            case ScrutariSource.DATA_NOTXML_STATE:
                return "state_data_notxml";
            default:
                throw new SwitchException("state = " + state);
        }
    }

    public static String getStateMessageKey(short state) {
        switch (state) {
            case ScrutariSource.OK_STATE:
                return "_ state.source.ok";
            case ScrutariSource.UNKNOWN_STATE:
                return "_ state.source.unknown";
            case ScrutariSource.INFO_NOCONNECTION_STATE:
                return "_ state.source.info_noconnection";
            case ScrutariSource.INFO_BROKENURL_STATE:
                return "_ state.source.info_brokenurl";
            case ScrutariSource.INFO_DOWNLOADERROR_STATE:
                return "_ state.source.info_downloaderror";
            case ScrutariSource.INFO_NOTXML_STATE:
                return "_ state.source.info_notxml";
            case ScrutariSource.INFO_WRONGXML_STATE:
                return "_ state.source.info_wrongxml";
            case ScrutariSource.DATA_NOCONNECTION_STATE:
                return "_ state.source.data_noconnection";
            case ScrutariSource.DATA_BROKENURL_STATE:
                return "_ state.source.data_brokenurl";
            case ScrutariSource.DATA_DOWNLOADERROR_STATE:
                return "_ state.source.data_downloaderror";
            case ScrutariSource.DATA_NOTXML_STATE:
                return "_ state.source.data_notxml";
            default:
                throw new SwitchException("state = " + state);
        }
    }

    public static short stateToShort(String stateString) {
        if (stateString.equals("state_ok")) {
            return ScrutariSource.OK_STATE;
        } else if (stateString.equals("state_unknown")) {
            return ScrutariSource.UNKNOWN_STATE;
        } else if (stateString.equals("state_info_noconnection")) {
            return ScrutariSource.INFO_NOCONNECTION_STATE;
        } else if (stateString.equals("state_info_brokenurl")) {
            return ScrutariSource.INFO_BROKENURL_STATE;
        } else if (stateString.equals("state_info_downloaderror")) {
            return ScrutariSource.INFO_DOWNLOADERROR_STATE;
        } else if (stateString.equals("state_info_notxml")) {
            return ScrutariSource.INFO_NOTXML_STATE;
        } else if (stateString.equals("state_info_wrongxml")) {
            return ScrutariSource.INFO_WRONGXML_STATE;
        } else if (stateString.equals("state_data_noconnection")) {
            return ScrutariSource.DATA_NOCONNECTION_STATE;
        } else if (stateString.equals("state_data_brokenurl")) {
            return ScrutariSource.DATA_BROKENURL_STATE;
        } else if (stateString.equals("state_data_downloaderror")) {
            return ScrutariSource.DATA_DOWNLOADERROR_STATE;
        } else if (stateString.equals("state_data_notxml")) {
            return ScrutariSource.DATA_NOTXML_STATE;
        }
        throw new IllegalArgumentException("Unknown state = " + stateString);
    }


    private static class InternalTrustManager implements X509TrustManager {

        private InternalTrustManager() {
        }

        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType) {
        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType) {
        }

        @Override
        public X509Certificate[] getAcceptedIssuers() {
            return null;
        }

    }


    private static class InternalCollectResult implements CollectResult {

        private final short state;
        private final int lastUpdate;

        private InternalCollectResult(short state, int lastUpdate) {
            this.state = state;
            this.lastUpdate = lastUpdate;
        }

        @Override
        public short getState() {
            return state;
        }

        @Override
        public int getLastUpdateDateInteger() {
            return lastUpdate;
        }

    }

}
