/* SctServer - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import java.net.URL;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariSourceDefBuilder {

    private final String name;
    private short urlType;
    private URL url;
    private int frequency = 1;

    public ScrutariSourceDefBuilder(String name) {
        if (!StringUtils.isTechnicalName(name, true)) {
            throw new IllegalArgumentException("Not a technical name: " + name);
        }
        this.name = name;
    }

    public void setScrutariDataUrl(URL url, int frequency) {
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        this.urlType = ScrutariSourceDef.SCRUTARIDATA_TYPE;
        this.url = url;
        if (frequency > 1) {
            this.frequency = frequency;
        } else {
            this.frequency = 1;
        }
    }

    public void setInfoUrl(URL url) {
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        this.urlType = ScrutariSourceDef.INFO_TYPE;
        this.url = url;
        this.frequency = 1;
    }

    public ScrutariSourceDef toScrutariSourceDef() {
        if (url == null) {
            throw new IllegalStateException("url is not initialized");
        }
        return new InternalScrutariSourceDef(name, urlType, url, frequency);
    }


    private static class InternalScrutariSourceDef implements ScrutariSourceDef {

        private final String name;
        private final short urlType;
        private final URL url;
        private final int frequency;

        private InternalScrutariSourceDef(String name, short urlType, URL url, int frequency) {
            this.name = name;
            this.urlType = urlType;
            this.url = url;
            this.frequency = frequency;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public short getUrlType() {
            return urlType;
        }

        @Override
        public URL getUrl() {
            return url;
        }

        @Override
        public int getFrequency() {
            return frequency;
        }

    }

}
