/* SctServer - Copyright (c) 2006-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.tools.ErrorUtils;
import java.text.ParseException;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.tools.options.CategoryDefBuilder;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CategoriesDOMReader {

    private final Builder builder;
    private final ProcessMessageHandler errorHandler;

    public CategoriesDOMReader(Builder builder, ProcessMessageHandler errorHandler) {
        this.builder = builder;
        this.errorHandler = errorHandler;
    }

    public void readCategories(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("category")) {
                    addCategory(element);
                }
            }
        }
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("corpus-list")) {
                    addCorpusList(element);
                } else if (!tagName.equals("category")) {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }

    private void addCategory(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, element_xml.getTagName(), "name");
            return;
        }
        CategoryDefBuilder categoryDefBuilder = builder.addCategory(name);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        Libelle libelle = LibelleUtils.readLib(element);
                        if (libelle != null) {
                            categoryDefBuilder.addLibelle(libelle);
                        }
                    } catch (ParseException lie) {
                    }
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }

    }

    private void addCorpusList(Element element_xml) {
        String name = element_xml.getAttribute("category-name");
        if (name.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, element_xml.getTagName(), "category-name");
            return;
        }
        if (name.equals(CategoryDef.DEFAULT_NAME)) {
            errorHandler.addMessage(SctLogConstants.WARNING_CATEGORY, "corpusListNotAllowed", name);
            return;
        }
        CategoryDefBuilder categoryDefBuilder = builder.getCategory(name);
        if (categoryDefBuilder == null) {
            errorHandler.addMessage(SctLogConstants.SEVERE_CATEGORY, "unknownCategory", name);
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    String uriString = XMLUtils.getData(element);
                    if (uriString.length() == 0) {
                        ErrorUtils.emptyElementError(errorHandler, "uri");
                        continue;
                    }
                    try {
                        ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
                        if (scrutariDataURI instanceof CorpusURI) {
                            boolean done = builder.addCorpusURI(categoryDefBuilder, (CorpusURI) scrutariDataURI);
                            if (!done) {
                                errorHandler.addMessage(SctLogConstants.WARNING_CATEGORY, "corpusURIDoublon", uriString);
                            }
                        }
                    } catch (URIParseException uripe) {
                        ErrorUtils.uriSyntaxError(errorHandler, uriString);
                    }
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, tagName);
                }
            }
        }
    }


    public static interface Builder {

        public CategoryDefBuilder addCategory(String name);

        public CategoryDefBuilder getCategory(String name);

        public boolean addCorpusURI(CategoryDefBuilder builder, CorpusURI corpusURI);

    }

}
