/* SctServer - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.OptionsDOMReader;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.SimpleProcessMessageHandler;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
class OptionsConf {

    private PertinencePonderation globalPertinencePonderation;
    private String processURI;
    private ProcessMessageList processMessageList;

    PertinencePonderation getGlobalPertinencePonderation() {
        return globalPertinencePonderation;
    }

    static OptionsConf load(File f) throws IOException, ParserConfigurationException {
        OptionsConf conf = new OptionsConf();
        conf.init(f);
        return conf;
    }

    String getProcessURI() {
        return processURI;
    }

    ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleProcessMessageHandler simpleHandler = new SimpleProcessMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(simpleHandler, saxe);
            processMessageList = simpleHandler.toList();
            return;
        }
        OptionsBuilder builder = new OptionsBuilder();
        OptionsDOMReader optionsDOMReader = new OptionsDOMReader(builder, simpleHandler);
        optionsDOMReader.readOptions(document.getDocumentElement());
        globalPertinencePonderation = builder.globalPertinencePonderation;
        processMessageList = simpleHandler.toList();
    }


    private static class OptionsBuilder implements OptionsDOMReader.Builder {

        PertinencePonderation globalPertinencePonderation;

        private OptionsBuilder() {

        }

        @Override
        public void setGlobalPertinencePonderation(PertinencePonderation pertinencePonderation) {
            this.globalPertinencePonderation = pertinencePonderation;
        }

    }
}
