/* SctServer - Copyright (c) 2009-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.tools.conf.dom.SourcesDOMReader;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.SimpleProcessMessageHandler;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
class SourcesConf {

    private ScrutariSourceDef[] scrutariSourceDefArray = null;
    private String processURI;
    private ProcessMessageList processMessageList;

    private SourcesConf() {
    }

    String getProcessURI() {
        return processURI;
    }

    ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    ScrutariSourceDef[] getScrutariSourceDefArray() {
        return scrutariSourceDefArray;
    }

    static SourcesConf load(File f) throws IOException, ParserConfigurationException {
        SourcesConf conf = new SourcesConf();
        conf.init(f);
        return conf;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        scrutariSourceDefArray = null;
        processURI = f.getPath();
        SimpleProcessMessageHandler simpleHandler = new SimpleProcessMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(simpleHandler, saxe);
            processMessageList = simpleHandler.toList();
            return;
        }
        SourcesDOMReader domReader = new SourcesDOMReader(simpleHandler);
        scrutariSourceDefArray = domReader.readSources(document.getDocumentElement());
        processMessageList = simpleHandler.toList();
    }

}
