/* SctServer - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.UriCodesDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.ProcessMessageHandler;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.SimpleProcessMessageHandler;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.codes.ScrutariDataURICodeList;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
final class UriCodesConf {

    private ScrutariDataURICodeList scrutariDataURICodeList;
    private String processURI;
    private ProcessMessageList processMessageList;

    private UriCodesConf() {
    }

    ScrutariDataURICodeList getScrutariDataURICodeList() {
        return scrutariDataURICodeList;
    }

    static UriCodesConf load(File f) throws IOException, ParserConfigurationException {
        UriCodesConf conf = new UriCodesConf();
        conf.init(f);
        return conf;
    }

    String getProcessURI() {
        return processURI;
    }

    ProcessMessageList getProcessMessageList() {
        return processMessageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleProcessMessageHandler simpleHandler = new SimpleProcessMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.saxException(simpleHandler, saxe);
            processMessageList = simpleHandler.toList();
            return;
        }
        UriCodesBuilder builder = new UriCodesBuilder(simpleHandler);
        UriCodesDOMReader domReader = new UriCodesDOMReader(builder, simpleHandler);
        domReader.readUriCodes(document.getDocumentElement());
        scrutariDataURICodeList = builder.toScrutariDataURICodeList();
        processMessageList = simpleHandler.toList();
    }


    private static class UriCodesBuilder implements UriCodesDOMReader.Builder {

        private final ProcessMessageHandler errorHandler;
        private final Map<Integer, ScrutariDataURI> codeMap = new LinkedHashMap<Integer, ScrutariDataURI>();
        private final Map<ScrutariDataURI, Integer> uriMap = new HashMap<ScrutariDataURI, Integer>();

        public UriCodesBuilder(ProcessMessageHandler errorHandler) {
            this.errorHandler = errorHandler;
        }

        @Override
        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI) {
            if (codeMap.containsKey(code)) {
                errorHandler.addMessage(SctLogConstants.WARNING_CATEGORY, "codeDoublon", new Integer(code));
                return;
            }
            if (uriMap.containsKey(scrutariDataURI)) {
                errorHandler.addMessage(SctLogConstants.WARNING_CATEGORY, "uriDoublon", scrutariDataURI);
                return;
            }
            codeMap.put(code, scrutariDataURI);
            uriMap.put(scrutariDataURI, code);
        }

        public ScrutariDataURICodeList toScrutariDataURICodeList() {
            int size = codeMap.size();
            if (size == 0) {
                return null;
            }
            int[] codeArray = new int[size];
            ScrutariDataURI[] uriArray = new ScrutariDataURI[size];
            int p = 0;
            for (Map.Entry<Integer, ScrutariDataURI> entry : codeMap.entrySet()) {
                codeArray[p] = entry.getKey();
                uriArray[p] = entry.getValue();
                p++;
            }
            return new ArrayScrutariDataURICodeList(codeArray, uriArray);
        }

    }


    private static class ArrayScrutariDataURICodeList implements ScrutariDataURICodeList {

        private final int[] codeArray;
        private final ScrutariDataURI[] uriArray;

        public ArrayScrutariDataURICodeList(int[] codeArray, ScrutariDataURI[] uriArray) {
            this.codeArray = codeArray;
            this.uriArray = uriArray;
        }

        @Override
        public int getCodeCount() {
            return codeArray.length;
        }

        @Override
        public int getCode(int i) {
            return codeArray[i];
        }

        @Override
        public ScrutariDataURI getScrutariDataURI(int i) {
            return uriArray[i];
        }

    }

}
