/* SctServer - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.log;

import fr.exemole.sctserver.api.collect.CollectErrorHandler;
import fr.exemole.sctserver.api.log.SctLogConstants;
import java.net.URL;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.ProcessMessageLogBuilder;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.xml.ScrutariXmlErrorHandler;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class UpdateLogHandler implements ScrutariXmlErrorHandler, CollectErrorHandler {

    private final ProcessMessageLogBuilder builder = new ProcessMessageLogBuilder();
    private String currentKey;
    private int keyOccurrence = 0;

    public UpdateLogHandler() {
    }

    public void setCurrentScrutariSource(String name) {
        flush();
        builder.setCurrentURI(name);
    }

    public void saxException(SAXException saxe) {
        flush();
        builder.addMessage(SctLogConstants.MALFORMED_XML_CATEGORY, SctLogConstants.SEVERE_DATA_BASEFILE_XML, saxe.getMessage());
    }

    public void xmlPullParserException(Exception xppe) {
        flush();
        builder.addMessage(SctLogConstants.MALFORMED_XML_CATEGORY, SctLogConstants.SEVERE_DATA_BASEFILE_XML, xppe.getMessage());
    }

    @Override
    public void addSevereLog(String key, ScrutariDataURI scrutariDataURI) {
        flush();
        builder.addMessage(SctLogConstants.SEVERE_CATEGORY, key, scrutariDataURI.toString());
    }

    @Override
    public void addWarningLog(String key, ScrutariDataURI scrutariDataURI) {
        flush();
        builder.addMessage(SctLogConstants.WARNING_CATEGORY, key, scrutariDataURI.toString());
    }

    @Override
    public void addSevereLog(String key) {
        flush();
        builder.addMessage(SctLogConstants.SEVERE_CATEGORY, key, null);
    }

    @Override
    public void addWarningLog(String key) {
        if ((currentKey != null) && (currentKey.equals(key))) {
            keyOccurrence++;
        } else {
            flush();
            currentKey = key;
            keyOccurrence = 1;
        }

    }

    @Override
    public void warningDownload(URL url, String message) {
        flush();
        builder.addMessage(SctLogConstants.WARNING_CATEGORY, "warning.download", message);
    }

    public String toContentString() {
        flush();
        return LogUtils.toString(builder.toProcessMessageLog());
    }

    private void flush() {
        if (currentKey != null) {
            builder.addMessage(SctLogConstants.WARNING_CATEGORY, currentKey, new Integer(keyOccurrence));
            keyOccurrence = 0;
            currentKey = null;
        }
    }

}
