/* UtilLib - Copyright (c) 2011-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.text.ParseException;
import net.mapeadores.util.text.StringUtils;


/**
 * [_a-zA-Z][-_.a-zA-Z0-9]*
 *
 * @author Vincent Calame
 */
public final class CheckedKeyString {

    private static final CheckedKeyString EMPTY_NAMESPACE = new CheckedKeyString("");
    private final String s;

    private CheckedKeyString(String s) {
        this.s = s;
    }

    @Override
    public String toString() {
        return s;
    }

    public boolean isEmpty() {
        return s.isEmpty();
    }

    public static CheckedKeyString newInstance(CharSequence charSequence) {
        return newInstance(charSequence, false);
    }

    public static CheckedKeyString newInstance(CharSequence charSequence, boolean intern) {
        if (charSequence == null) {
            return EMPTY_NAMESPACE;
        }
        String s = StringUtils.cleanString(charSequence);
        if (s.length() == 0) {
            return EMPTY_NAMESPACE;
        }
        if (intern) {
            s = s.intern();
        }
        return new CheckedKeyString(s);
    }

    public static CheckedKeyString parse(CharSequence charSequence) throws ParseException {
        int length = charSequence.length();
        if (length == 0) {
            throw new ParseException("Empty charSequence", 0);
        }
        if (!testFirstChar(charSequence.charAt(0))) {
            throw new ParseException("Wrong first char (" + charSequence.charAt(0) + ")", 0);
        }
        for (int i = 1; i < length; i++) {
            char carac = charSequence.charAt(i);
            if (!testChar(carac)) {
                throw new ParseException("Wrong char (" + carac + ")", i);
            }
        }
        return new CheckedKeyString(charSequence.toString());
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt
     * que ParseException. à utiliser quand on est sûr de la syntaxe et permet
     * d'éviter un try {} catch {}
     */
    public static CheckedKeyString build(CharSequence charSequence) {
        try {
            return parse(charSequence);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    private static boolean testFirstChar(char carac) {
        if (carac == '_') {
            return true;
        }
        if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        }
        if ((carac >= 'A') && (carac <= 'Z')) {
            return true;
        }
        return false;
    }

    private static boolean testChar(char carac) {
        if (carac == '_') {
            return true;
        }
        if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        }
        if ((carac >= 'A') && (carac <= 'Z')) {
            return true;
        }
        if ((carac >= '0') && (carac <= '9')) {
            return true;
        }
        switch (carac) {
            case '_':
            case '-':
            case '.':
                return true;
            default:
                return false;
        }
    }

}
