/* UtilLib - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCondition implements Condition, ConditionConstants {

    private final short mode;
    private final List<TextTest> excludingList = new ArrayList<TextTest>();
    private final List<TextTest> includingList = new ArrayList<TextTest>();
    private short state = U;

    public DefaultCondition(short mode) {
        this.mode = mode;
    }

    @Override
    public short getMode() {
        return mode;
    }

    @Override
    public short getState() {
        return state;
    }

    @Override
    public int getExcludingTextTestCount() {
        return excludingList.size();
    }

    @Override
    public int getIncludingTextTestCount() {
        return includingList.size();
    }

    @Override
    public TextTest getExcludingTextTest(int i) {
        return excludingList.get(i);
    }

    @Override
    public TextTest getIncludingTextTest(int i) {
        return includingList.get(i);
    }

    protected void innerSetState(short state) {
        this.state = state;
    }

    protected void innerAddTextTest(TextTest textTest) {
        short testType = textTest.getTestType();
        short addingState = ConditionUtils.getState(testType);
        state = ConditionUtils.merge(mode, state, addingState);
        if (!ConditionUtils.isPartState(state)) {
            excludingList.clear();
            includingList.clear();
        } else {
            if (ConditionUtils.isExcludingPartTestType(testType)) {
                if (excludingList.size() < 31) {
                    excludingList.add(new DefaultTextTest(textTest)); //@todo cover
                }
            } else if (ConditionUtils.isIncludingPartTestType(testType)) {
                if (includingList.size() < 31) {
                    includingList.add(new DefaultTextTest(textTest)); //@todo cover
                }
            }
        }
    }

    public static Condition build(TextTest textTest) {
        DefaultCondition conditionBuilder = new DefaultCondition(ConditionConstants.MODE_AND);
        conditionBuilder.innerAddTextTest(textTest);
        return conditionBuilder;
    }

}
