/* UtilLib - Copyright (c) 2011-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.format;


/**
 *
 * @author Vincent Calame
 */
public final class FormatUtils implements FormatConstants {

    private FormatUtils() {
    }

    public static short getCastType(FormatDef formatDef) {
        return toCastType(formatDef.getParameterValue(CAST_PARAMKEY));
    }

    public static short toCastType(Object value) {
        if (value == null) {
            return FormatConstants.NO_CAST;
        }
        try {
            return ((Short) value);
        } catch (ClassCastException cce) {
            return FormatConstants.NO_CAST;
        }
    }

    public static int getMaxLength(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(MAXLENGTH_PARAMKEY);
        if (value == null) {
            return -1;
        }
        return ((Integer) value);
    }

    public static int getFixedLength(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(FIXEDLENGTH_PARAMKEY);
        if (value == null) {
            return -1;
        }
        return ((Integer) value);
    }

    public static char getFixedChar(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(FIXEDCHAR_PARAMKEY);
        if (value == null) {
            return ' ';
        }
        return ((Character) value);
    }

    public static boolean isFixedEmpty(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(FIXEDEMPTY_PARAMKEY);
        if (value == null) {
            return false;
        }
        return ((Boolean) value);
    }

    public static boolean isGlobalSelect(FormatDef formatDef, boolean defaultGlobalSelect) {
        Object value = formatDef.getParameterValue(GLOBALSELECT_PARAMKEY);
        if (value == null) {
            return defaultGlobalSelect;
        }
        return ((Boolean) value);
    }

    public static boolean isConvertEmptyToNull(FormatDef formatDef) {
        return toConvertEmptyToNull(formatDef.getParameterValue(EMPTYTONULL_PARAMKEY));
    }

    public static boolean toConvertEmptyToNull(Object value) {
        if (value == null) {
            return false;
        }
        try {
            return ((Boolean) value);
        } catch (ClassCastException cce) {
            return false;
        }
    }

    public static String getDefaultValue(FormatDef formatDef) {
        return (String) formatDef.getParameterValue(DEFAULTVALUE_PARAMKEY);
    }

    public static int getLimit(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(LIMIT_PARAMKEY);
        if (value == null) {
            return -1;
        }
        return ((Integer) value);
    }

    public static int getPosition(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(POSITION_PARAMKEY);
        if (value == null) {
            return -1;
        }
        return ((Integer) value);
    }

    public static Calcul getCalcul(FormatDef formatDef) {
        return toCalcul(formatDef.getParameterValue(CALCUL_PARAMKEY));
    }

    public static Calcul toCalcul(Object value) {
        if (value == null) {
            return null;
        }
        try {
            return (Calcul) value;
        } catch (ClassCastException cce) {
            return null;
        }
    }

    public static boolean isWithSum(FormatDef formatDef) {
        return (formatDef.getParameterValue(SUM_PARAMKEY) != null);
    }

    public static short getSumCastType(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(SUM_PARAMKEY);
        if (value == null) {
            return FormatConstants.NO_CAST;
        }
        return ((Short) value);
    }

    public static boolean isWithColumnSum(FormatDef formatDef) {
        return toWithColumnSum(formatDef.getParameterValue(COLUMNSUM_PARAMKEY));
    }

    public static boolean toWithColumnSum(Object value) {
        if (value == null) {
            return false;
        }
        try {
            return ((Boolean) value);
        } catch (ClassCastException cce) {
            return false;
        }
    }

    public static String getSeparator(FormatDef formatDef, String defaultSeparator) {
        if (isWithSum(formatDef)) {
            return ";";
        }
        Object value = formatDef.getParameterValue(SEPARATOR_PARAMKEY);
        if (value == null) {
            return defaultSeparator;
        }
        return (String) value;
    }

    public static String getInnerSeparator(FormatDef formatDef, int sourceIndex, String defaultSeparator) {
        if (isWithSum(formatDef)) {
            return ";";
        }
        FormatDef.InnerSeparator sep = getInnerSeparatorObject(formatDef, sourceIndex);
        if (sep == null) {
            return getSeparator(formatDef, defaultSeparator);
        } else {
            return sep.getSeparator();
        }
    }

    public static String getPosttransform(FormatDef formatDef) {
        return (String) formatDef.getParameterValue(POSTTRANSFORM_PARAMKEY);
    }

    public static String getPrefix(FormatDef formatDef) {
        return (String) formatDef.getParameterValue(PREFIX_PARAMKEY);
    }

    public static String getSuffix(FormatDef formatDef) {
        return (String) formatDef.getParameterValue(SUFFIX_PARAMKEY);
    }

    private static FormatDef.InnerSeparator getInnerSeparatorObject(FormatDef formatDef, int sourceIndex) {
        int count = formatDef.getInnerSeparatorCount();
        if (count == 0) {
            return null;
        }
        for (int i = 0; i < count; i++) {
            FormatDef.InnerSeparator innerSeparator = formatDef.getInnerSeparator(i);
            if (innerSeparator.getSourceIndex() == sourceIndex) {
                return innerSeparator;
            }
        }
        return null;
    }

    public static String getSourceSeparator(FormatDef formatDef, int sourceIndex1, int sourceIndex2, String defaultSeparator) {
        if (isWithSum(formatDef)) {
            return ";";
        }
        FormatDef.SourceSeparator sep = getSourceSeparatorObject(formatDef, sourceIndex1, sourceIndex2);
        if (sep == null) {
            return getSeparator(formatDef, defaultSeparator);
        } else {
            return sep.getSeparator();
        }
    }

    private static FormatDef.SourceSeparator getSourceSeparatorObject(FormatDef formatDef, int sourceIndex1, int sourceIndex2) {
        int count = formatDef.getSourceSeparatorCount();
        if (count == 0) {
            return null;
        }
        if (sourceIndex1 > sourceIndex2) {
            int tmp = sourceIndex1;
            sourceIndex1 = sourceIndex2;
            sourceIndex2 = tmp;
        }
        FormatDef.SourceSeparator nearestSourceSeparator = null;
        for (int i = 0; i < count; i++) {
            FormatDef.SourceSeparator sourceSeparator = formatDef.getSourceSeparator(i);
            if (sourceSeparator.getSourceIndex1() == sourceIndex1) {
                int si2 = sourceSeparator.getSourceIndex2();
                if (si2 == sourceIndex2) {
                    return sourceSeparator;
                } else if (si2 > sourceIndex2) {
                    continue;
                } else {
                    if ((nearestSourceSeparator == null) || (nearestSourceSeparator.getSourceIndex2() < si2)) {
                        nearestSourceSeparator = sourceSeparator;
                    }
                }
            }
        }
        if (nearestSourceSeparator != null) {
            return nearestSourceSeparator;
        }
        return null;
    }

    public static short castTypeToShort(String s) {
        if ((s.equals(INTEGER_PARAMVALUE)) || (s.equals("int"))) {
            return INTEGER_CAST;
        }
        if ((s.equals(DECIMAL_PARAMVALUE)) || (s.equals("double")) || (s.equals("float"))) {
            return DECIMAL_CAST;
        }
        if (s.equals(DATE_PARAMVALUE)) {
            return DATE_CAST;
        }
        if (s.equals(MONEY_PARAMVALUE)) {
            return MONEY_CAST;
        }
        throw new IllegalArgumentException("Unknown cast type : " + s);
    }

    public static String castTypeToString(short castType) {
        if (castType == INTEGER_CAST) {
            return INTEGER_PARAMVALUE;
        }
        if (castType == DECIMAL_CAST) {
            return DECIMAL_PARAMVALUE;
        }
        if (castType == DATE_CAST) {
            return DATE_PARAMVALUE;
        }
        if (castType == MONEY_CAST) {
            return MONEY_PARAMVALUE;
        }
        throw new IllegalArgumentException("Unknown cast type : " + castType);
    }

}
