/* UtilLib - Copyright (c) 2012-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.Message;


/**
 *
 * @author Vincent Calame
 */
public class SimpleProcessMessageHandler implements ProcessMessageHandler {

    private final List<ProcessMessage> list = new ArrayList<ProcessMessage>();

    public SimpleProcessMessageHandler() {
    }

    @Override
    public void addMessage(String category, String key, Object value) {
        list.add(LogUtils.newProcessMessage(category, key, value));
    }

    @Override
    public void addMessage(String category, Message message) {
        addMessage(category, message.getMessageKey(), message.getMessageValue());
    }

    public boolean hasMessage() {
        return (!list.isEmpty());
    }

    public ProcessMessageList toList() {
        return LogUtils.toProcessMessageList(list);
    }

    public ProcessMessageList filter(String category) {
        List<ProcessMessage> result = new ArrayList<ProcessMessage>();
        for (ProcessMessage processMessage : list) {
            if (processMessage.getCategory().equals(category)) {
                result.add(processMessage);
            }
        }
        return LogUtils.toProcessMessageList(result);
    }

    @Override
    public String toString() {
        StringBuilder buf = new StringBuilder();
        for (ProcessMessage processMessage : list) {
            if (buf.length() > 0) {
                buf.append("\n");
            }
            buf.append(processMessage.getMessageKey());
            Object value = processMessage.getMessageValue();
            if (value != null) {
                if (value instanceof Object[]) {
                    Object[] array = (Object[]) value;
                    int length = array.length;
                    for (int i = 0; i < length; i++) {
                        if (i > 0) {
                            buf.append(" / ");
                        } else {
                            buf.append(": ");
                        }
                        buf.append(array[i].toString());
                    }
                } else {
                    buf.append(": ");
                    buf.append(value.toString());
                }

            }
        }
        return buf.toString();
    }

}
