/* UtilLib - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.request;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.ChangeAttributeHolderBuilder;


/**
 *
 * @author Vincent Calame
 */
public class RequestConfBuilder {

    private Map<String, InternalSupplementaryParameter> map = new HashMap<String, InternalSupplementaryParameter>();
    private InternalSupplementaryParameterList supplementaryParameterList = new InternalSupplementaryParameterList();
    private ChangeAttributeHolderBuilder attributeHolderBuilder = new ChangeAttributeHolderBuilder();

    public RequestConf toRequestConf() {
        return new InternalRequestConf(supplementaryParameterList, attributeHolderBuilder.toChangeAttributeHolder());
    }

    public void addParameter(String name, Collection<String> values) {
        if (values.size() == 0) {
            return;
        }
        InternalSupplementaryParameter supplementaryParameter = map.get(name);
        if (supplementaryParameter == null) {
            supplementaryParameter = new InternalSupplementaryParameter(name);
            map.put(name, supplementaryParameter);
            supplementaryParameterList.add(supplementaryParameter);
        }
        supplementaryParameter.add(values);
    }

    public void putAttribute(Attribute attribute) {
        attributeHolderBuilder.putAttribute(attribute);
    }


    private static class InternalRequestConf implements RequestConf {

        private SupplementaryParameterList supplementaryParameterList;
        private AttributeHolder attributeHolder;

        private InternalRequestConf(SupplementaryParameterList supplementaryParameterList, AttributeHolder attributeHolder) {
            this.supplementaryParameterList = supplementaryParameterList;
            this.attributeHolder = attributeHolder;
        }

        public SupplementaryParameterList getSupplementaryParameterList() {
            return supplementaryParameterList;
        }

        public AttributeHolder getAttributes() {
            return attributeHolder;
        }

    }


    private static class InternalSupplementaryParameterList implements SupplementaryParameterList {

        private List<InternalSupplementaryParameter> list = new ArrayList<InternalSupplementaryParameter>();

        public int getSupplementaryParameterCount() {
            return list.size();
        }

        public SupplementaryParameter getSupplementaryParameter(int i) {
            return list.get(i);
        }

        private void add(InternalSupplementaryParameter supplementaryParameter) {
            list.add(supplementaryParameter);
        }

    }


    private static class InternalSupplementaryParameter implements SupplementaryParameter {

        private String name;
        private boolean append = false;
        private List<String> valueList = new ArrayList();

        private InternalSupplementaryParameter(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public boolean appendToExisting() {
            return append;
        }

        public boolean isArray() {
            return (valueList.size() == 1);
        }

        public String getValue() {
            return valueList.get(0);
        }

        public String[] getValueArray() {
            return valueList.toArray(new String[valueList.size()]);
        }

        private void add(Collection<String> values) {
            valueList.addAll(values);
        }

    }

}
