/* UtilLib - Copyright (c) 2011-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;


/**
 *
 * @author Vincent Calame
 */
public class ChangeLibelleHolderBuilder {

    private final Set<Lang> removedLangSet = new LinkedHashSet<Lang>();
    private final Map<Lang, Libelle> libelleMap = new LinkedHashMap<Lang, Libelle>();

    public ChangeLibelleHolderBuilder() {

    }

    public void putLibelle(Libelle libelle) {
        Lang lang = libelle.getLang();
        libelleMap.put(lang, libelle);
        removedLangSet.remove(lang);
    }

    public Libelle putLibelle(Lang lang, CharSequence charSequence) {
        return putLibelle(lang, CleanedString.newInstance(charSequence));
    }

    public Libelle putLibelle(Lang lang, CleanedString cleanedString) {
        if (cleanedString == null) {
            removedLangSet.add(lang);
            libelleMap.remove(lang);
            return null;
        } else {
            Libelle libelle = LibelleUtils.toLibelle(lang, cleanedString);
            libelleMap.put(lang, libelle);
            removedLangSet.remove(lang);
            return libelle;
        }
    }

    public Libelle getLibelle(Lang lang) {
        return libelleMap.get(lang);
    }

    public void putRemovedLang(Lang lang) {
        removedLangSet.add(lang);
        libelleMap.remove(lang);
    }

    public void merge(ChangeLibelleHolderBuilder otherChangeLibelleHolderBuilder) {
        for (Lang otherLang : otherChangeLibelleHolderBuilder.removedLangSet) {
            if (!libelleMap.containsKey(otherLang)) {
                removedLangSet.add(otherLang);
            }
        }
        for (Map.Entry<Lang, Libelle> entry : otherChangeLibelleHolderBuilder.libelleMap.entrySet()) {
            Lang otherLang = entry.getKey();
            if (!libelleMap.containsKey(otherLang)) {
                libelleMap.put(otherLang, entry.getValue());
                removedLangSet.remove(otherLang);
            }
        }
    }

    public ChangeLibelleHolder toChangeLibelleHolder() {
        Lang[] removedLangArray = removedLangSet.toArray(new Lang[removedLangSet.size()]);
        Libelle[] libelleArray = libelleMap.values().toArray(new Libelle[libelleMap.size()]);
        return new InternalChangeLibelleHolder(libelleMap, libelleArray, removedLangArray);
    }


    private static class InternalChangeLibelleHolder implements ChangeLibelleHolder {

        private final Lang[] removedLangArray;
        private final Libelle[] libelleArray;
        private final Map<Lang, Libelle> libelleMap;

        private InternalChangeLibelleHolder(Map<Lang, Libelle> libelleMap, Libelle[] libelleArray, Lang[] removedLangArray) {
            this.removedLangArray = removedLangArray;
            this.libelleArray = libelleArray;
            this.libelleMap = libelleMap;
        }

        @Override
        public int getLibelleCount() {
            return libelleArray.length;
        }

        @Override
        public Libelle getLibelleByIndex(int i) {
            return libelleArray[i];
        }

        @Override
        public Libelle getLibelleByLang(Lang lang) {
            return libelleMap.get(lang);
        }

        @Override
        public int getRemovedLangCount() {
            return removedLangArray.length;
        }

        @Override
        public Lang getRemovedLang(int i) {
            return removedLangArray[i];
        }

    }

}
