/* UtilLib - Copyright (c) 2007-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;


/**
 *
 * @author Vincent Calame
 */
public class DefaultLibelleHolder implements LibelleHolder {

    private final Map<Lang, Libelle> libelleMap = new HashMap<Lang, Libelle>();
    private final List<Libelle> libelleList = new ArrayList<Libelle>();

    public DefaultLibelleHolder() {
    }

    @Override
    public Libelle getLibelleByLang(Lang lang) {
        return (Libelle) libelleMap.get(lang);
    }

    @Override
    public int getLibelleCount() {
        return libelleList.size();
    }

    @Override
    public Libelle getLibelleByIndex(int i) {
        return libelleList.get(i);
    }

    protected boolean innerPutLibelle(Libelle libelle) {
        Lang lang = libelle.getLang();
        Libelle oldLibelle = getLibelleByLang(lang);
        if (oldLibelle == null) {
            libelleMap.put(lang, libelle);
            libelleList.add(libelle);
            return true;
        } else {
            if (LibelleUtils.areEquals(oldLibelle, libelle)) {
                return false;
            }
            int idx = libelleList.indexOf(oldLibelle);
            libelleMap.put(lang, libelle);
            libelleList.set(idx, libelle);
            return true;
        }
    }

    protected boolean innerRemoveLibelle(Lang lang) {
        Libelle oldLibelle = libelleMap.remove(lang);
        if (oldLibelle != null) {
            libelleList.remove(oldLibelle);
            return true;
        } else {
            return false;
        }
    }

    protected void clear() {
        libelleMap.clear();
        libelleList.clear();
    }

}
