/* UtilLib - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml.xmlpull;

import java.text.ParseException;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class LibelleElementHandler implements XmlPullElementHandler {

    private final static Lang UNDETERMINED_LANG = Lang.build("und");
    private Lang lang;
    private final StringBuilder buf = new StringBuilder();
    private int subCount = 0;

    public LibelleElementHandler(Map<String, String> attributeMap) throws ParseException {
        if (attributeMap != null) {
            String langIntegerString = (String) attributeMap.get("xml:lang");
            if ((langIntegerString != null) && (langIntegerString.length() > 0)) {
                lang = Lang.parse(langIntegerString);
            }
        }
    }

    @Override
    public void processStartElement(String tagname, Map<String, String> attributeMap) {
        subCount++;
        buf.append("<");
        buf.append(tagname);
        buf.append(">");
    }

    @Override
    public boolean processEndElement(String tagname) {
        if (subCount == 0) {
            return true;
        }
        subCount--;
        buf.append("</");
        buf.append(tagname);
        buf.append(">");
        return false;
    }

    @Override
    public void processText(String text) {
        buf.append(text);
    }

    public XmlPullElementHandler initSubElementHandler(String tagname, Map attributeMap) {
        return null;
    }

    public void flushSubElementHandler(String tagname, XmlPullElementHandler elementHandler) {
    }

    /**
     * Peut être nul
     */
    public Libelle toLibelle() {
        CleanedString cleanedString = CleanedString.newInstance(buf);
        if (cleanedString == null) {
            return null;
        }
        if (lang == null) {
            lang = UNDETERMINED_LANG;
        }
        return LibelleUtils.toLibelle(lang, cleanedString);
    }

}
