/* UtilLib_Servlet  - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets.handlers;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.MessageList;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.CommandMessageJsonProducer;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.mimetype.MimeTypeConstants;
import net.mapeadores.util.request.RequestConstants;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.request.RequestUtils;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessageResponseHandler implements ResponseHandler {

    private final CommandMessage commandMessage;
    private final MessageLocalisation messageLocalisation;
    private long lastModified = ResponseHandler.LASTMODIFIED_NOCACHE;

    private CommandMessageResponseHandler(CommandMessage commandMessage, MessageLocalisation messageLocalisation) {
        this.commandMessage = commandMessage;
        this.messageLocalisation = messageLocalisation;
    }

    @Override
    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(MimeTypeConstants.HTML + ";charset=UTF-8");
        PrintWriter pw = response.getWriter();
        HtmlPrinter htmlPrinter = new HtmlPrinter();
        htmlPrinter.initPrinter(pw);
        htmlPrinter.addMessageLocalisation(messageLocalisation);
        htmlPrinter.startHtml(null);
        htmlPrinter.startHead();
        htmlPrinter.startTitle();
        htmlPrinter.escape(commandMessage.getMessageKey());
        htmlPrinter.endTitle();
        htmlPrinter.endHead();
        htmlPrinter.startBody();
        htmlPrinter.startP();
        HTMLAttributes spanAttributes;
        if (commandMessage.isErrorMessage()) {
            spanAttributes = HA.style("color: red");
        } else {
            spanAttributes = HA.style("color: green");
        }
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.localize(commandMessage);
        htmlPrinter.endSpan();
        htmlPrinter.endP();
        if (commandMessage.hasMultiError()) {
            htmlPrinter.startUl();
            MessageList list = commandMessage.getMultiErrorList();
            int count = list.getMessageCount();
            for (int i = 0; i < count; i++) {
                htmlPrinter.startLi();
                htmlPrinter.localize(list.getMessage(i));
                htmlPrinter.endLi();
            }
            htmlPrinter.endUl();
        }
        if (commandMessage.hasLog()) {
            htmlPrinter.startPre(null);
            htmlPrinter.escape(LogUtils.toString(commandMessage.getProcessMessageLog()), true);
            htmlPrinter.endPre();
        }
        htmlPrinter.endBody();
        pw.close();
    }

    public static ResponseHandler getInstance(CommandMessage commandMessage, MessageLocalisation messageLocalisation, RequestMap requestMap) {
        if (RequestUtils.isJsonDefined(requestMap)) {
            return getJsonInstance(commandMessage, messageLocalisation, requestMap);
        }
        return getHtmlInstance(commandMessage, messageLocalisation);
    }

    public static ResponseHandler getJsonInstance(CommandMessage commandMessage, MessageLocalisation messageLocalisation, RequestMap requestMap) {
        JsonProducer jsonProducer = null;
        if (commandMessage != null) {
            jsonProducer = new CommandMessageJsonProducer(commandMessage, messageLocalisation);
        }
        short jsonType = RequestUtils.getJsonType(requestMap);
        return new JsonResponseHandler(jsonProducer, jsonType, requestMap.getParameter(RequestConstants.CALLBACK_PARAMETER));
    }

    public static ResponseHandler getHtmlInstance(CommandMessage commandMessage, MessageLocalisation messageLocalisation) {
        return new CommandMessageResponseHandler(commandMessage, messageLocalisation);
    }

}
